@@ -1,13 +1,12 @@
 #!/usr/bin/env python
 from __future__ import print_function, with_statement
 import contextlib
-import subprocess
 import os
+import subprocess
 import sys
 import warnings
 
 from docopt import docopt
-from django.core.management import CommandError
 from django.utils.encoding import force_text
 from django.utils.importlib import import_module
 
@@ -27,7 +26,7 @@ Usage:
     djangocms-helper <application> cms_check [--extra-settings=</path/to/settings.py>] [--migrate]
     djangocms-helper <application> compilemessages [--extra-settings=</path/to/settings.py>] [--cms]
     djangocms-helper <application> makemessages [--extra-settings=</path/to/settings.py>] [--cms]
-    djangocms-helper <application> makemigrations [--extra-settings=</path/to/settings.py>] [--cms] [--merge]
+    djangocms-helper <application> makemigrations [--extra-settings=</path/to/settings.py>] [--cms] [--merge] [<extra-applications>...]
     djangocms-helper <application> squashmigrations <migration-name>
     djangocms-helper <application> pyflakes [--extra-settings=</path/to/settings.py>] [--cms]
     djangocms-helper <application> authors [--extra-settings=</path/to/settings.py>] [--cms]
@@ -48,6 +47,7 @@ Options:
     --runner-options=<option1>,<option2>        Comma separated list of command line options for the test runner
     --port=<port>                               Port to listen on [default: 8000].
     --bind=<bind>                               Interface to bind to [default: 127.0.0.1].
+    <extra-applications>                        Comma separated list of applications to create migrations for
 '''
 
 
@@ -84,7 +84,10 @@ def test(test_labels, application, failfast=False, test_runner=None,
         if os.path.exists('tests'):
             test_labels = ['tests']
         elif os.path.exists(os.path.join(application, 'tests')):
-            test_labels = [application]
+            if 'DjangoTestSuiteRunner' in test_runner:
+                test_labels = [application]
+            else:
+                test_labels = ['%s.tests' % application]
     return _test_run_worker(test_labels, test_runner, failfast, runner_options)
 
 
@@ -139,33 +142,55 @@ def cms_check(migrate_cmd=False):
     call_command('cms', 'check')
 
 
-def makemigrations(application, merge=False):
+def makemigrations(application, merge=False, extra_applications=None):
     """
     Generate migrations (for both south and django 1.7+)
     """
+    from django.core.exceptions import DjangoRuntimeWarning, ImproperlyConfigured
     from django.core.management import call_command
-    from .utils import load_from_file
+    from .utils import captured_output
+
+    apps = [application]
+    if extra_applications:
+        apps += extra_applications
 
     if DJANGO_1_6:
-        try:
-            loaded = load_from_file(os.path.join(application, 'migrations',
-                                                 '0001_initial.py'))
-        except IOError:
-            loaded = None
-        initial = loaded is None
-        call_command('schemamigration', initial=initial, auto=(not initial),
-                     *(application,))
+        from south.exceptions import NoMigrations
+        from south.migration import Migrations
+
+        if merge:
+            raise DjangoRuntimeWarning(u'Option not implemented for Django 1.6 and below')
+        for app in apps:
+            try:
+                if not Migrations(app):
+                    raise NoMigrations(app)
+            except NoMigrations:
+                print('ATTENTION: No migrations found for {0}, creating initial migrations.'.format(app))
+                try:
+                    call_command('schemamigration', *(app,), initial=True)
+                except SystemExit:
+                    pass
+            except ImproperlyConfigured:
+                print('WARNING: The app: {0} could not be found.'.format(app))
+            else:
+                try:
+                    with captured_output():
+                        call_command('schemamigration', *(app,), auto=True)
+                except SystemExit:
+                    pass
     else:
-        call_command('makemigrations', *(application,), merge=merge)
+        for app in apps:
+            call_command('makemigrations', *(app,), merge=merge)
 
 
 def squashmigrations(application, migration):
     """
     Squash Django 1.7+ migrations
     """
+    from django.core.exceptions import DjangoRuntimeWarning
     from django.core.management import call_command
     if DJANGO_1_6:
-        raise CommandError(u'Command not implemented for Django 1.6')
+        raise DjangoRuntimeWarning(u'Command not implemented for Django 1.6 and below')
     else:
         call_command('squashmigrations', application, migration)
 
@@ -218,7 +243,7 @@ def static_analisys(application):
         print(u"Static analisys available only if django CMS is installed")
 
 
-def server(bind='127.0.0.1', port=8000, migrate_cmd=False):
+def server(bind='127.0.0.1', port=8000, migrate_cmd=False):  # pragma: no cover
     from cms.utils.compat.dj import get_user_model
     from django.utils import autoreload
 
@@ -269,7 +294,10 @@ def core(args, application):
                 elif args['--runner']:
                     runner = args['--runner']
                 else:
-                    runner = 'django.test.runner.DiscoverRunner'
+                    if DJANGO_1_5:
+                        runner = 'django.test.simple.DjangoTestSuiteRunner'
+                    else:
+                        runner = 'django.test.runner.DiscoverRunner'
                 # make "Address already in use" errors less likely, see Django
                 # docs for more details on this env variable.
                 os.environ.setdefault(
@@ -306,7 +334,7 @@ def core(args, application):
             elif args['makemessages']:
                 makemessages(application)
             elif args['makemigrations']:
-                makemigrations(application, merge=args['--merge'])
+                makemigrations(application, merge=args['--merge'], extra_applications=args['<extra-applications>'])
             elif args['squashmigrations']:
                 squashmigrations(application, args['<migration-name>'])
             elif args['pyflakes']:
