@@ -2,8 +2,11 @@
 from __future__ import print_function, with_statement
 from copy import copy
 from distutils.version import LooseVersion
+from django.utils.encoding import force_text
+import os
 import os.path
 import shutil
+import sys
 try:
     import unittest2 as unittest
 except ImportError:
@@ -13,7 +16,7 @@ from cms.test_utils.tmpdir import temp_dir
 import django
 
 from ..main import core, _make_settings
-from ..utils import work_in, captured_output
+from ..utils import work_in, captured_output, DJANGO_1_6, DJANGO_1_5
 
 DEFAULT_ARGS = {
     'shell': False,
@@ -35,7 +38,10 @@ DEFAULT_ARGS = {
     '--cms': True,
     '--failfast': False,
     '--merge': False,
-    '<test-label>': ''
+    '--bind': '',
+    '--port': '',
+    '<test-label>': '',
+    '<extra-applications>': ''
 }
 
 
@@ -44,31 +50,46 @@ class CommandTests(unittest.TestCase):
     basedir = None
     pofile = None
     mofile = None
+    migration_dir = None
 
     @classmethod
     def setUpClass(cls):
         cls.basedir = os.path.abspath(os.path.join('djangocms_helper', 'test_utils'))
-        cls.application = 'example'
+        cls.application = 'example1'
+        cls.application_2 = 'example2'
         with work_in(cls.basedir):
             with captured_output() as (out, err):
                 cls.poexample = os.path.abspath(os.path.join(cls.application, 'data', 'django.po'))
                 cls.pofile = os.path.abspath(os.path.join(cls.application, 'locale', 'en', 'LC_MESSAGES', 'django.po'))
                 cls.mofile = os.path.abspath(os.path.join(cls.application, 'locale', 'en', 'LC_MESSAGES', 'django.mo'))
                 cls.migration_dir = os.path.abspath(os.path.join(cls.application, 'migrations'))
+                cls.migration_dir_2 = os.path.abspath(os.path.join(cls.application_2, 'migrations'))
                 cls.migration_file = os.path.abspath(os.path.join(cls.application, 'migrations', '0001_initial.py'))
+                cls.migration_file_2 = os.path.abspath(os.path.join(cls.application_2, 'migrations', '0001_initial.py'))
 
     def setUp(self):
         try:
             os.unlink(self.pofile)
-        except OSError:
+        except (OSError, TypeError):
             pass
         try:
             os.unlink(self.mofile)
-        except OSError:
+        except (OSError, TypeError):
             pass
         try:
-            shutil.rmtree(self.migration_dir)
-        except OSError:
+            if self.migration_dir:
+                shutil.rmtree(self.migration_dir)
+            if self.migration_dir_2:
+                shutil.rmtree(self.migration_dir_2)
+        except (OSError, TypeError):
+            pass
+        try:
+            del sys.modules['example1.migrations']
+        except KeyError as e:
+            pass
+        try:
+            del sys.modules['example2.migrations']
+        except KeyError as e:
             pass
 
     def tearDown(self):
@@ -108,10 +129,68 @@ class CommandTests(unittest.TestCase):
             with captured_output() as (out, err):
                 args = copy(DEFAULT_ARGS)
                 args['makemigrations'] = True
-                args['--merge'] = True
+                args['<extra-applications>'] = ['example2']
                 core(args, self.application)
-                self.assertTrue(os.path.exists(self.migration_file))
+            self.assertTrue(os.path.exists(self.migration_file))
+            self.assertTrue(os.path.exists(self.migration_file_2))
+        if DJANGO_1_6:
+            self.assertTrue('Created 0001_initial.py' in err.getvalue())
+            self.assertTrue('migrate example1' in err.getvalue())
+            self.assertTrue('migrate example2' in err.getvalue())
+        else:
+            self.assertTrue('Create model ExampleModel1' in out.getvalue())
+            self.assertTrue('Create model ExampleModel2' in out.getvalue())
+
+    @unittest.skipIf(LooseVersion(django.get_version()) >= LooseVersion('1.7'),
+                     reason='check only for Django < 1.7')
+    def test_makemigrations_existing_folder(self):
+        os.makedirs(self.migration_dir)
+        os.makedirs(self.migration_dir_2)
+        open(os.path.join(self.migration_dir, '__init__.py'), 'w')
+        open(os.path.join(self.migration_dir_2, '__init__.py'), 'w')
+        with work_in(self.basedir):
+            with captured_output() as (out, err):
+                args = copy(DEFAULT_ARGS)
+                args['makemigrations'] = True
+                args['<extra-applications>'] = ['example2']
+                core(args, self.application)
+            self.assertTrue(os.path.exists(self.migration_file))
+            self.assertTrue(os.path.exists(self.migration_file_2))
         self.assertTrue('Created 0001_initial.py' in err.getvalue())
+        self.assertTrue('migrate example1' in err.getvalue())
+        self.assertTrue('migrate example2' in err.getvalue())
+
+    def test_makemigrations_merge(self):
+        from django.core.exceptions import DjangoRuntimeWarning
+        with work_in(self.basedir):
+            with captured_output() as (out, err):
+                args = copy(DEFAULT_ARGS)
+                args['makemigrations'] = True
+                args['--merge'] = True
+                if DJANGO_1_6:
+                    with self.assertRaises(DjangoRuntimeWarning) as exit:
+                        core(args, self.application)
+                    self.assertEqual(force_text(exit.exception), 'Option not implemented for Django 1.6 and below')
+                else:
+                    core(args, self.application)
+                    self.assertTrue('No conflicts detected to merge' in out.getvalue())
+
+    def test_squashmigrations(self):
+        from django.core.exceptions import DjangoRuntimeWarning
+        from django.core.management import CommandError
+        with work_in(self.basedir):
+            with captured_output() as (out, err):
+                args = copy(DEFAULT_ARGS)
+                args['squashmigrations'] = True
+                args['<migration-name>'] = '0001_initial'
+                if DJANGO_1_6:
+                    with self.assertRaises(DjangoRuntimeWarning) as exit:
+                        core(args, self.application)
+                    self.assertEqual(force_text(exit.exception), 'Command not implemented for Django 1.6 and below')
+                else:
+                    with self.assertRaises(CommandError) as exit:
+                        core(args, self.application)
+                    self.assertTrue('squashmigrations on it makes no sense' in force_text(exit.exception))
 
     def test_makemessages(self):
         with work_in(self.basedir):
@@ -139,7 +218,7 @@ class CommandTests(unittest.TestCase):
                 args = copy(DEFAULT_ARGS)
                 args['check'] = True
                 core(args, self.application)
-            self.assertEqual(len(out.getvalue()), 0)
+        self.assertTrue('no issues' in out.getvalue())
 
     def test_cms_check(self):
         with work_in(self.basedir):
@@ -160,6 +239,8 @@ class CommandTests(unittest.TestCase):
                 args['pyflakes'] = True
                 core(args, self.application)
 
+    @unittest.skipIf(sys.version_info < (2, 7),
+                     reason="Example test non discoverable in Python 2.6")
     def test_testrun(self):
         with work_in(self.basedir):
             with captured_output() as (out, err):
@@ -168,9 +249,11 @@ class CommandTests(unittest.TestCase):
                     args['test'] = True
                     args['--runner'] = 'runners.CapturedOutputRunner'
                     core(args, self.application)
-        self.assertTrue('Ran 1 test in 0.000s' in err.getvalue())
+        self.assertTrue('Ran 3 tests in' in err.getvalue())
         self.assertEqual(exit.exception.code, 0)
 
+    @unittest.skipIf(sys.version_info < (2, 7),
+                     reason="Example test non discoverable in Python 2.6")
     def test_testrun_nocms(self):
         with work_in(self.basedir):
             with captured_output() as (out, err):
@@ -180,7 +263,7 @@ class CommandTests(unittest.TestCase):
                     args['--cms'] = False
                     args['--runner'] = 'runners.CapturedOutputRunner'
                     core(args, self.application)
-        self.assertTrue('Ran 1 test in 0.000s' in err.getvalue())
+        self.assertTrue('Ran 3 tests in' in err.getvalue())
         self.assertEqual(exit.exception.code, 0)
 
     def test_authors(self):
@@ -191,7 +274,7 @@ class CommandTests(unittest.TestCase):
                 core(args, self.application)
         self.assertTrue('Generating AUTHORS' in out.getvalue())
         self.assertTrue('* Iacopo Spalletti' in out.getvalue())
-        self.assertTrue('Authors (2):' in out.getvalue())
+        self.assertTrue('Authors (' in out.getvalue())
 
     def test_urls(self):
         from django.core.urlresolvers import reverse
